#!/bin/sh

if [ $# -eq 1 ]; then
if [ -d $1 ]; then
if [ -e $1/tomcat/webapps/aipo/WEB-INF/conf/AipoResources.properties ]; then
parent_dir=$1



#######################################################################
#
#######################################################################
. /etc/sysconfig/network-scripts/ifcfg-eth0
addr=$IPADDR
if [ "$addr" = "" ]; then
	addr=`ifconfig eth0 | grep "inet addr" | cut -f 2 -d :| cut -f 1 -d " "`
fi

#############################################
## 設定に必要な変数をセット
#############################################
#Aipoバージョン
aipo_version="4.0.5.0"
aipo_version_int="4050"
# 最大ループ回数
Max_Loop="100"

#ユーザ確認
current_user=`whoami`
if [ "${current_user}" != "root" ]; then
    echo "Aipo$aipo_versionのインストールはroot権限で行ってください。"
    exit 0
fi

#Old Version
tmp_str=`cat ${parent_dir}/tomcat/webapps/aipo/WEB-INF/conf/AipoResources.properties | grep aipo.version=`
old_version=`echo "$tmp_str" | cut -f 2 -d "="`
isCR=`expr match $old_version .*[0-9]$ `
if [ "$isCR" = "0" ]; then
    len=`expr length $old_version`
    len=`expr $len - 1`
    old_version=`expr substr $old_version 1 $len`
fi
echo $old_version

#バックアップフォルダ作成
mkdir -p ${script_path}/backup
bk_dir=${script_path}/backup

#######################################################################
#復元
#######################################################################
function restorefunc
{
    alias cp='cp'
    if [ -d $bk_dir/tomcat/ ]; then
       if [ -e $parent_dir/tomcat/webapps ]; then
            mv $parent_dir/tomcat/webapps $parent_dir/tomcat/webappsbk${date_str}
       fi
       cp -rf  $bk_dir/tomcat/webapps $parent_dir/tomcat/
    fi

    sudo -u $postgres_user $parent_dir/postgres/bin/pg_ctl -D $parent_dir/postgres/data -o "-S -i" stop

    echo "Aipo$aipo_versionへのアップデートに失敗しました。"
    alias cp='cp -i'
    exit 1
}

function restoreDBfunc
{
    echo "データベースの復元を開始します。"
    #org001をリネームしてorg001failedを作成
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001bk${date_str} -c "ALTER DATABASE org001 RENAME TO org001failed${date_str};"
    #org001bk${date_str}をコピーしてorg001を復元
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001bk${date_str} -c "CREATE DATABASE org001 TEMPLATE org001bk${date_str};"
    if [ $? != 0 ]; then
       echo "データベースの復元に失敗しました。"
       restorefunc
    fi
    echo "データベースの復元を終了しました。"
    restorefunc
}

#Aipoバージョン確認
if [ "$old_version" = "$aipo_version" ]; then
	echo "既にAipo$aipo_versionがインストールされています。"
	exit 0
fi

if [ "${old_version}" != "4.0.4.0" ]; then
if [ "${old_version}" != "4.0.3.0" ]; then
if [ "${old_version}" != "4.0.2.0" ]; then
if [ "${old_version}" != "4.0.1.0" ]; then
if [ "${old_version}" != "4.0.0.0" ]; then
	echo "Aipo$old_version　に対応していません。"
	exit 0
fi
fi
fi
fi
fi

tmp_str=`cat $parent_dir/bin/startup.sh | grep "s_port_num="`
s_port_num=`echo "$tmp_str" | cut -f 2 -d "="`

#以前のバージョンのPostgreSQLユーザー情報を取得
echo "PostgreSQLのユーザー情報を取得しています"
tmp_str=`cat $parent_dir/tomcat/webapps/aipo/WEB-INF/conf/postgres/Org001DomainNode.driver.xml | grep login`
postgres_user=`echo "$tmp_str" | cut -f 2 -d " " | cut -f 2 -d = | cut -f 2 -d \"`
echo user:$postgres_user
pass=`echo "$tmp_str" | cut -f 3 -d " " | cut -f 2 -d = | cut -f 1 -d \\/ | cut -f 2 -d \"`
echo passwd:$pass

tmp_str=`cat $parent_dir/tomcat/webapps/aipo/WEB-INF/conf/postgres/Org001DomainNode.driver.xml | grep url`
port_num=`echo "$tmp_str" | cut -f 4 -d ":" | cut -f 1 -d \\/ `
echo port:$port_num

unset tmp_str

sh $parent_dir/bin/shutdown.sh
#fuser -vn tcp $port_num
#if [ $? = 0 ]; then
#	echo "Postmasterが起動中です。Postmasterを一旦停止させてからアップデートをやり直してください。"
#	exit 0
#fi

#PostgreSQLの起動
sudo -u $postgres_user $parent_dir/postgres/bin/postmaster -D $parent_dir/postgres/data -S -i
#wait
prc_name="$parent_dir/postgres/bin/postmaster"
count=0
while [ $count -lt $Max_Loop ]; do
	tmp_str=`ps auxww | grep $prc_name`
	tmp_str=`echo $tmp_str`
	pid_str=`echo "$tmp_str" | cut -f 2 -d " "`
	tmp_str=`netstat -nap | grep "ESTABLISHED $pid_str"`
	if [ "$tmp_str" != "" ]; then
		echo $tmp_str
		break
	fi
	count=`expr $count + 1`
done



#################################################
##ファイルのコピー
#################################################

#バックアップ
mkdir -p $bk_dir/tomcat/
cp -rf $parent_dir/tomcat/webapps $bk_dir/tomcat/
#データベースのコピー
date_str=`date +"%Y%m%d%H%M"`
sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -c "CREATE DATABASE org001bk${date_str} TEMPLATE org001;"
#pgpassの更新
sudo -u ${postgres_user} echo "localhost:$port_num:org001bk${date_str}:${postgres_user}:$pass" >> /home/${postgres_user}/.pgpass
chown ${postgres_user}:${postgres_user} /home/${postgres_user}/.pgpass
sudo -u ${postgres_user} chmod 0600 /home/${postgres_user}/.pgpass

#コピー
alias cp='cp'
tmp_version=$old_version
if [ "$tmp_version" = "4.0.0.0" ]; then
	echo "Update from Aipo 4.0.0.0 ..."
	cp -rf ${script_path}/aipo4010/dpl003/tomcat/webapps $parent_dir/tomcat/
	cp -rf ${script_path}/aipo4010/dpl003/tomcat/webapps/aipo/WEB-INF/conf/*.xreg $parent_dir/backup/xreg

	#characteristic code for 4.0.0.0
	sed -i "s/aipo.webappserver.port=8080/aipo.webappserver.port=${s_port_num}/g" $parent_dir/tomcat/webapps/aipo/WEB-INF/conf/AipoResources.properties

	#UPDATE DATA BASE By PSQL
	cd ${script_path}/aipo4010/bin
	sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f aipo4010update.sql

	#UPDATE DATA BASE By JAVA
	$parent_dir/jre/bin/java -cp ".":"$parent_dir/tomcat/webapps/aipo/WEB-INF/lib/aipo-4.0.jar":"$parent_dir/tomcat/webapps/aipo/WEB-INF/lib/castor-0.9.3.jar":"$parent_dir/tomcat/webapps/aipo/WEB-INF/lib/xercesImpl-2.0.2.jar":"$parent_dir/tomcat/webapps/aipo/WEB-INF/lib/log4j-1.2.9.jar":"$parent_dir/tomcat/common/lib/postgresql-8.1-407.jdbc3.jar" com.aimluck.eip.update.DBConverter4000to4010 dbname=org001 port=$port_num user=$postgres_user passwd=$pass home=$parent_dir "" "$parent_dir/logs/pg_update.log"

	if [ $? != 0 ]; then
	   restorefunc
	fi
	tmp_version="4.0.1.0"
fi


if [ "$tmp_version" = "4.0.1.0" ]; then
    echo "Update from Aipo 4.0.1.0 ..."
    cp -rf ${script_path}/aipo4020/dpl003/tomcat/webapps $parent_dir/tomcat/
    cp -rf ${script_path}/aipo4020/dpl003/tomcat/webapps/aipo/WEB-INF/conf/*.xreg $parent_dir/backup/xreg

    #UPDATE DATA BASE By PSQL
    cd ${script_path}/aipo4020/bin
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f aipo4020update.sql

    if [ $? != 0 ]; then
       restorefunc
    fi

    #characteristic code for 4.0.1.0
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -c "UPDATE EIP_M_COMPANY SET PORT_INTERNAL=${s_port_num}"

    if [ $? != 0 ]; then
       restorefunc
    fi
    tmp_version="4.0.2.0"
fi


if [ "$tmp_version" = "4.0.2.0" ]; then
    echo "Update from Aipo 4.0.2.0 ..."
    cp -rf ${script_path}/aipo4030/dpl003/tomcat/webapps $parent_dir/tomcat/
    cp -rf ${script_path}/aipo4030/dpl003/tomcat/webapps/aipo/WEB-INF/conf/*.xreg $parent_dir/backup/xreg
    rm -rf $parent_dir/tomcat/webapps/aipo/WEB-INF/conf/access_ctl.xreg
    rm -rf $parent_dir/backup/xreg/access_ctl.xreg

    if [ $? != 0 ]; then
       restorefunc
    fi

    #UPDATE PSML By JAVA
    cd ${script_path}/aipo4030/bin
    $parent_dir/jre/bin/java -cp ".":"../lib/aipo-4.0.jar":"../lib/turbine-2.2-al1.jar":"../lib/cayenne-2.0.4.jar":"../lib/castor-0.9.3.jar":"../lib/xercesImpl-2.0.2.jar":"../lib/log4j-1.2.9.jar" com.aimluck.eip.update.UpdatePsml4020to4030 "$parent_dir/tomcat/webapps" "" "$parent_dir/logs/pg_update.log"

    if [ $? != 0 ]; then
       restorefunc
    fi
    tmp_version="4.0.3.0"
fi

if [ "$tmp_version" = "4.0.3.0" ]; then
    echo "Update from Aipo 4.0.3.0 ..."
    cp -rf ${script_path}/aipo4040/dpl003/tomcat/webapps $parent_dir/tomcat/
    cp -rf ${script_path}/aipo4040/dpl003/tomcat/webapps/aipo/WEB-INF/conf/*.xreg $parent_dir/backup/xreg

    #UPDATE DATA BASE By PSQL
    cd ${script_path}/aipo4040/bin
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f aipo4040update.sql

    if [ $? != 0 ]; then
       restorefunc
    fi

    #※この下はaipo4040update.sqlで最新のDBに代わったため、restoreにはrestoreDBfuncを使用すること

    #過去の不要なテーブルをDROP
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop1.sql 2>/dev/null
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop2.sql 2>/dev/null
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop3.sql 2>/dev/null
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop4.sql 2>/dev/null
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop5.sql 2>/dev/null
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop6.sql 2>/dev/null
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f drop7.sql 2>/dev/null

    #テーブル構造検証
    echo "テーブル構造の検証を開始します。"
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f ${script_path}/aipo_common/bin/select_tables.sql > ${script_path}/check_table.txt

    if [ $? != 0 ] || [ ! -e ${script_path}/check_table.txt ]; then
        echo "テーブル構造の検証に失敗しました。"
        restoreDBfunc
    fi

    diff_result=`diff -w table_dump4040 ${script_path}/check_table.txt`
    if [ "$diff_result" != "" ]; then
        echo "データベースの更新に失敗した可能性があります。"
        restoreDBfunc
    fi
    echo "テーブル構造の検証を完了しました。"
    rm ${script_path}/check_table.txt

    tmp_version="4.0.4.0"
fi

if [ "$tmp_version" = "4.0.4.0" ]; then
    echo "Update from Aipo 4.0.4.0 ..."
    cp -rf ${script_path}/aipo4050/dpl003/tomcat/webapps $parent_dir/tomcat/
    cp -rf ${script_path}/aipo4050/dpl003/tomcat/webapps/aipo/WEB-INF/conf/*.xreg $parent_dir/backup/xreg

    #UPDATE DATA BASE By PSQL
    cd ${script_path}/aipo4050/bin
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f aipo4050update.sql

    if [ $? != 0 ]; then
       restorefunc
    fi

    #※この下はaipo4050update.sqlで最新のDBに代わったため、restoreにはrestoreDBfuncを使用すること

    #テーブル構造検証
    echo "テーブル構造の検証を開始します。"
    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f ${script_path}/aipo_common/bin/select_tables.sql > ${script_path}/check_table.txt

    if [ $? != 0 ] || [ ! -e ${script_path}/check_table.txt ]; then
        echo "テーブル構造の検証に失敗しました。"
        restoreDBfunc
    fi

    diff_result=`diff -w table_dump4050 ${script_path}/check_table.txt`
    if [ "$diff_result" != "" ]; then
        echo "データベースの更新に失敗した可能性があります。"
        restoreDBfunc
    fi
    echo "テーブル構造の検証を完了しました。"
    rm ${script_path}/check_table.txt

    tmp_version="4.0.5.0"
fi

# Update Template
# if [ "$tmp_version" = "4.x.x.x" ]; then
#
#    echo "Update from Aipo 4.x.x.x ..."
#    cp -rf ${script_path}/aipoyyyy/dpl003/tomcat/webapps $parent_dir/tomcat/
#    cp -rf ${script_path}/aipoyyyy/dpl003/license $parent_dir
#    cp -rf ${script_path}/aipoyyyy/dpl003/tomcat/webapps/aipo/WEB-INF/conf/*.xreg $parent_dir/backup/xreg
#    cp -rf ${script_path}/aipoyyy/dpl003/src/bin $parent_dir/backup/xreg
#
#    #UPDATE DATA BASE By PSQL
#    cd ${script_path}/aipoyyyy/bin
#    sudo -u $postgres_user $parent_dir/postgres/bin/psql org001 -f aipo4yyyyupdate.sql
#   tmp_version="4.y.y.y"
# fi

if [ $tmp_version != $aipo_version ]; then
    restorefunc
fi

cp -rf ${script_path}/aipo${aipo_version_int}/dpl003/license/license.txt $parent_dir
alias cp='cp -i'

############################################
#実行用スクリプト生成
############################################

echo "$parent_dir" | sed s/'\/'/'\\\/'/g > temp.txt
tmp_str=`cat temp.txt`
rm -rf temp.txt
echo $tmp_str

mkdir -p $bk_dir/bin
cp -rf $parent_dir/bin/*.sh $bk_dir/bin/

cd ${script_path}/aipo${aipo_version_int}/dpl003/src/bin/${enc_str}
for FILE in *.sh; do
	mv -f $parent_dir/bin/$FILE $bk_dir/bin/$FILE
	sed "s/default_folder/$tmp_str/g" ${script_path}/aipo${aipo_version_int}/dpl003/src/bin/${enc_str}/$FILE | sed "s/default_user/$postgres_user/g" | sed "s/default_server_port/$s_port_num/g" | sed "s/default_db_port/$port_num/g" | sed "s/postgres_pass/$pass/g" | sed "s/Aipo_Version/$aipo_version/g" > $parent_dir/bin/$FILE
done

chmod +x $parent_dir/bin/*.sh

###################################################
##終了処理
###################################################
sudo -u $postgres_user $parent_dir/postgres/bin/pg_ctl -D $parent_dir/postgres/data -o "-S -i" stop
#wait

######################################
#結果表示
######################################
echo ==Aipo"$aipo_version"アップデート終了==
echo user:$postgres_user
echo pass:$pass
echo directory:$parent_dir
echo port:$port_num
echo "Aipo URL: http://$addr:$s_port_num/aipo/"

else
 echo "The input directory is wrong."
fi

else
 echo "No folder."
fi

else
 echo "Please input the directory of aipo."
fi